package fr.plnech.dunbar

import android.content.ContentUris
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.provider.ContactsContract
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.content_main.*
import java.io.IOException


class MainActivity : AppCompatActivity() {

    private val friends = mutableListOf<Friend>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        setSupportActionBar(toolbar)

        fab.setOnClickListener { view ->
            reloadFriends()
        }

        reloadFriends()
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        // Inflate the menu; this adds items to the action bar if it is present.
        menuInflater.inflate(R.menu.menu_main, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        return when (item.itemId) {
            R.id.action_settings -> true
            else -> super.onOptionsItemSelected(item)
        }
    }

    private fun reloadFriends() {
        fetchFriends()
        displayFriends()

        Toast.makeText(this, "Friends reloaded.", Toast.LENGTH_SHORT).show()
    }

    private fun fetchFriends() {
        val newFriends = mutableListOf<Friend>()

        contentResolver.query(
            ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
            null,
            null,
            null,
            null
        )?.let {
            while (it.moveToNext()) {
                mutableMapOf<String, String?>().also { map ->
                    val indexId = it.getColumnIndex(ContactsContract.Contacts._ID)
                    val id = it.getLong(indexId)
                    val photo: Bitmap? = getPhoto(id)

                    it.columnNames.forEach { name ->
                        val index = it.getColumnIndex(name)
                        map[name] = it.getString(index)
                    }
                    newFriends.add(Friend(map, photo))
                }
            }
            it.close()

            friends.clear()
            friends.addAll(newFriends)
        }

        friends.sortByDescending { it.timesContacted }
    }

    private fun displayFriends() {
        val adapter = FriendsAdapter(friends)

        friendsList.layoutManager = LinearLayoutManager(this)
        friendsList.setHasFixedSize(true)
        friendsList.adapter = adapter
//        friendsList.setOnItemClickListener { parent, view, position, id ->
//            val contact: String = adapter.getItem(position)?.toString() ?: "NONE"
//
//                Snackbar.make(friendsList, contact, Snackbar.LENGTH_INDEFINITE).show()
//            Toast.makeText(this, contact, Toast.LENGTH_LONG).show()
//        }

        val nbFriends = adapter.itemCount
        welcomeTitle.text = "$nbFriends ${"friend".plural(nbFriends)} on Dunbar"

        Toast.makeText(this@MainActivity, "$nbFriends ", Toast.LENGTH_LONG).show()
    }

    private fun getPhoto(id: Long): Bitmap? {
        var photo: Bitmap? = null

        try {
            val inputStream = ContactsContract.Contacts.openContactPhotoInputStream(
                contentResolver, ContentUris.withAppendedId(
                    ContactsContract.Contacts.CONTENT_URI,
                    java.lang.Long.valueOf(id)
                )
            )

            if (inputStream != null) {
                photo = BitmapFactory.decodeStream(inputStream)
            }

            inputStream?.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
        Log.d("--> ", photo.toString())
        return photo
    }
}
